<?php
/**
 * Add custom checkout fields in WooCommerce
 *
 * @author ilGhera
 * @version 1.0.5
 */
class WCEXD_Checkout_Fields {

	public $custom_fields;

	public function __construct() {

		add_action( 'wp_enqueue_scripts', array( $this, 'add_checkout_script' ) );
		add_filter( 'woocommerce_checkout_fields', array( $this, 'set_custom_fields' ) );
		add_action( 'woocommerce_checkout_create_order', array( $this, 'save_fields' ), 10, 2 );
		add_action( 'woocommerce_thankyou', array( $this, 'display_custom_data' ) );
		add_action( 'woocommerce_view_order', array( $this, 'display_custom_data' ) );
		add_action( 'woocommerce_admin_order_data_after_billing_address', array( $this, 'display_custom_data_in_admin' ) );
		add_filter( 'woocommerce_email_customer_details', array( $this, 'display_custom_data_in_email' ), 10, 4 );
		add_action( 'woocommerce_checkout_process', array( $this, 'checkout_fields_check' ) );
		add_action( 'show_user_profile', array( $this, 'extra_user_profile_fields' ) );
		add_action( 'edit_user_profile', array( $this, 'extra_user_profile_fields' ) );
		add_action( 'personal_options_update', array( $this, 'save_extra_user_profile_fields' ) );
		add_action( 'edit_user_profile_update', array( $this, 'save_extra_user_profile_fields' ) );

		$this->custom_fields = $this->get_active_custom_fields();
		$this->cf_mandatory  = get_option( 'wcexd_cf_mandatory' );
		$this->piva_only_ue  = get_option( 'wcexd_piva_only_ue' );
		$this->only_italy    = get_option( 'wcexd_only_italy' );
		$this->cf_only_italy = get_option( 'wcexd_cf_only_italy' );
	    $this->ue            = array( 'AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'EL', 'ES', 'FI', 'FR', 'HR', 'HU', 'IE', 'IT', 'LU', 'LV', 'LT', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK' );


	}

	/**
	 * Caricamento script
	 */
	public function add_checkout_script() {

		wp_enqueue_script( 'wcexd-checkout-script', WCEXD_URI . 'includes/wc-checkout-fields/js/wcexd-checkout.js' );
		wp_localize_script(
			'wcexd-checkout-script',
			'options',
			array(
				'cf_mandatory'  => $this->cf_mandatory,
				'piva_only_ue'  => $this->piva_only_ue,
				'ue'            => $this->ue,
				'only_italy'    => $this->only_italy,
				'cf_only_italy' => $this->cf_only_italy,
			)
		);

	}


	/**
	 * Campi da aggiungere alla pagina di checkout in base alle opzioni scelte dall'utente
	 */
	public function get_active_custom_fields() {

		$output = array();

		$custom_fields = array(
			'billing_wcexd_piva'    => __( 'Partita IVA', 'wcexd' ),
			'billing_wcexd_cf'      => __( 'Codice fiscale', 'wcexd' ),
			'billing_wcexd_pec'     => __( 'PEC', 'wcexd' ),
			'billing_wcexd_pa_code' => __( 'Codice destinatario', 'wcexd' ),
		);

		foreach ( $custom_fields as $key => $value ) {
			if ( get_option( $key . '_active' ) === '1' ) {
				$output[ $key ] = $value;
			}
		}

		return $output;
	}


	/**
	 * Aggiunge i field customizzati all'elenco fields di WC
	 *
	 * @param object $fields i campi del modulo di checkout già presenti.
	 */
	public function set_custom_fields( $fields ) {

		$select = array(
			'private' => array(
				'active' => get_option( 'wcexd_private' ),
				'field'  => array( 'private' => __( 'Privato ( Ricevuta )', 'wcexd' ) ),
			),
			'private_invoice' => array(
				'active' => get_option( 'wcexd_private_invoice' ),
				'field' => array( 'private-invoice' => __( 'Privato ( Fattura )', 'wcexd' ) ),
			),
			'company_invoice' => array(
				'active' => get_option( 'wcexd_company_invoice' ),
				'field' => array( 'company-invoice' => __( 'Azienda ( Fattura )', 'wcexd' ) ),
			),
		);

		/*La somma dei tipi di documenti abilitati dall'admin*/
		$sum = ( $select['private']['active'] + $select['private_invoice']['active'] + $select['company_invoice']['active'] );

		if ( $sum > 1 ) {
			$fields['billing']['billing_wcexd_invoice_type'] = array(
				'type'    => 'select',
				'options' => array(),
				'label'   => __( 'Documento fiscale', 'wcexd' ),
				'required'    => true,
				'class'   => array(
					'field-name form-row-wide',
				),
			);

			/*Controllo posizione campo*/
			if ( get_option( 'wcexd_document_type' ) ) {

				$fields['billing']['billing_wcexd_invoice_type']['priority'] = 1;

			}

			foreach ( $select as $key => $value ) {
				if ( '1' === $value['active'] ) {
					$label = key( $value['field'] );
					$fields['billing']['billing_wcexd_invoice_type']['options'][ $label ] = $value['field'][ $label ];
				}
			}
		}

		if ( ! empty( $this->custom_fields ) ) {
			foreach ( $this->custom_fields as $key => $value ) {

                /* Input type */
                $input_type = 'billing_wcexd_pec' === $key ? 'email' : 'text';

				$fields['billing'][ $key ] = array(
					'type' => $input_type,
					'label' => $value,
					'class' => array(
						'field-name form-row-wide',
					),
				);
			}

			if ( isset( $this->custom_fields['billing_wcexd_piva'] ) ) {
				$fields['billing']['billing_wcexd_piva']['required'] = true;
			}

			/*Obbligatorietà cf al caricamento di pagina*/
			if ( isset( $this->custom_fields['billing_wcexd_cf'] ) ) {
				
				if ( $select['private_invoice']['active'] ) {

					$fields['billing']['billing_wcexd_cf']['required'] = true;
			
				} elseif ( $select['private']['active'] && in_array( $this->cf_mandatory, array( 1, 3 ) ) ) {

					$fields['billing']['billing_wcexd_cf']['required'] = true;

				
				} elseif ( $select['company_invoice']['active'] && in_array( $this->cf_mandatory, array( 2, 3 ) ) ) {

					$fields['billing']['billing_wcexd_cf']['required'] = true;

				}

			}

			/*Rendo obbligatorio cf e p. iva ed azienda solo quando richiesto*/
			if ( isset( $_POST['billing_wcexd_invoice_type'] ) ) {

				/* Reset */
				$fields['billing']['billing_wcexd_cf']['required'] = false;

				switch ( $_POST['billing_wcexd_invoice_type'] ) {
					
					case 'private':

						$fields['billing']['billing_wcexd_piva']['required'] = false;

						if ( in_array( $this->cf_mandatory, array( 1, 3 ) ) ) {

							$fields['billing']['billing_wcexd_cf']['required'] = true;

						}

						break;
					
					case 'private-invoice':

						$fields['billing']['billing_wcexd_piva']['required'] = false;
						$fields['billing']['billing_wcexd_cf']['required'] = true;

						break;

					case 'company-invoice':

						$fields['billing']['billing_company']['required'] = true;

						if ( in_array( $this->cf_mandatory, array( 2, 3 ) ) ) {

							$fields['billing']['billing_wcexd_cf']['required'] = true;

						}

						break;

				}

			}

			/* Ordine estero */
			if ( isset( $_POST['billing_country'] ) && 'IT' !== $_POST['billing_country'] ) {

				/* Codice fiscale non obbligatorio fuori dall'Italia */
				$fields['billing']['billing_wcexd_cf']['required'] = false;

				/* Se impostato, P.IVA non obbligatoria fuori dall'UE */
				if ( $this->piva_only_ue && isset( $this->custom_fields['billing_wcexd_piva'] ) ) {

					if ( ! in_array( $_POST['billing_country'] , $this->ue ) ) {

						$fields['billing']['billing_wcexd_piva']['required'] = false;

					}
				
				}


			}

			/*Rendo obbligatorio cf e p. iva ed azienda solo quando richiesto*/
			if ( isset( $_POST['billing_wcexd_invoice_type'] ) ) {

				if ( 'private' !== $_POST['billing_wcexd_invoice_type'] ) {

					if ( ! isset( $this->custom_fields['billing_wcexd_pec'] ) && isset( $this->custom_fields['billing_wcexd_pa_code'] ) ) {

						$fields['billing']['billing_wcexd_pa_code']['required'] = true;

					} elseif ( isset( $this->custom_fields['billing_wcexd_pec'] ) && ! isset( $this->custom_fields['billing_wcexd_pa_code'] ) ) {

						$fields['billing']['billing_wcexd_pec']['required'] = true;

					}
				}
			}
		}
		
		return $fields;

	}


	/**
	 * Verifica la correttezza del dato fiscale inserito
	 *
	 * @param  string $valore P.IVA o Codice Fiscale.
	 * @return bool
	 */
	public function fiscal_field_checker( $valore ) {
		$expression = '^[a-zA-Z]{6}[0-9]{2}[a-zA-Z][0-9]{2}[a-zA-Z][0-9]{3}[a-zA-Z]$';
		
		/* Controllo Partita IVA */
		if ( is_numeric( $valore ) ) {
			$expression = '^[0-9]{11}$';
		}

		/* Controllo Codice Fiscale */
		if ( preg_match( '/' . $expression . '/', $valore ) ) {
			return true;
		}
		return false;
	}


	/**
	 * Controllo VIES Partita IVA
	 *
	 * @param  int $piva    il numero di Partita IVA da controllare.
	 * @param  int $country lo stato di appartenenza del cliente.
	 *
	 * @return bool
	 */
	public function vies_piva_check( $piva, $country ) {

	    $output = true;
	    
	    if ( in_array( $country , $this->ue ) ) {

            /* Escludo controllo VIES per l'Italia */
            if ( 'IT' === $country ) {

                $output = $this->fiscal_field_checker( $piva );

            } else {

                $client  = new SoapClient('http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl');
                $check   = $client->checkVat(
                    array(
                        'countryCode' => $country,
                        'vatNumber'   => $piva,
                    )
                );

                $output = $check->valid;

            }


	    }

		return $output;

	}

    /*
     * Elimino campi non previsti dal tipo di documento fiscale selezionato
     *
     * @param string $document_type Il tipo di documento fiscale selezionato dall'utente.
     *
     * @return void
     */
    public function clean_data( $document_type ) {

        if ( $document_type && 'company-invoice' !== $document_type ) {

            if ( isset( $_POST['billing_company'] ) ) {

                unset( $_POST['billing_company'] );

            }

            if ( isset( $_POST['billing_wcexd_piva'] ) ) {

                unset( $_POST['billing_wcexd_piva'] );

            }

            if ( 'private' === $document_type ) {

                if ( isset( $_POST['billing_wcexd_pec'] ) ) {

                    unset( $_POST['billing_wcexd_pec'] );

                }

                if ( isset( $_POST['billing_wcexd_pa_code'] ) ) {

                    unset( $_POST['billing_wcexd_pa_code'] );

                }

            }

        }

    }


	/**
	 * Verifica i campi di checkout al click per la creazione dell'ordine
	 */
	public function checkout_fields_check() {
				
		$country       = sanitize_text_field( wp_unslash( $_POST['billing_country'] ) );
        $document_type = isset( $_POST['billing_wcexd_invoice_type'] ) ? sanitize_text_field( wp_unslash( $_POST['billing_wcexd_invoice_type']) ) : null;

        /* Controllo PEC */
        if ( isset( $this->custom_fields['billing_wcexd_pec'] ) ) {

            $pec = isset( $_POST['billing_wcexd_pec'] ) ? sanitize_text_field( wp_unslash( $_POST['billing_wcexd_pec'] ) ) : '';

            if ( $pec && ! is_email( $pec ) ) {

                wc_add_notice( __( 'ATTENZIONE! Sembra che la <strong>PEC</strong> inserita non sia corretta.', 'wcexd' ), 'error' );

            }
        }

		/* Controllo Codice Fiscale */
		if ( get_option( 'wcexd_fields_check' ) ) {

			/*Codice fiscale*/
			if ( isset( $_POST['billing_wcexd_cf'] ) && '' !== $_POST['billing_wcexd_cf'] && false === $this->fiscal_field_checker( $_POST['billing_wcexd_cf'] ) ) {

				wc_add_notice( __( 'ATTENZIONE! Sembra che il <strong>Codice Fiscale</strong> inserito non sia corretto.', 'wcexd' ), 'error' );

			}

		}

		/* Controllo VIES Partita IVA */
		if ( get_option( 'wcexd_vies_check' ) ) {

			/*Partita IVA*/
			if ( 'company-invoice' === $document_type ) {
					
				if ( isset( $_POST['billing_wcexd_piva'] ) && '' !== $_POST['billing_wcexd_piva'] && false === $this->vies_piva_check( $_POST['billing_wcexd_piva'], $country ) ) {

					wc_add_notice( __( 'ATTENZIONE! Sembra che la <strong>Partita IVA</strong> inserita non sia corretta.', 'wcexd' ), 'error' );

				}
			}

		}

        /* Elimino i dati non richiesti */
        $this->clean_data( $document_type );

	}


	/**
	 * Salva i campi personalizzati
	 *
	 * @param  object $order l'ordine in questione.
	 * @param  array  $data  i dati dell'ordine.
	 */
	public function save_fields( $order, $data ) {

		if ( $this->custom_fields ) {
			foreach ( $this->custom_fields as $key => $value ) {

                if ( 'billing_wcexd_pa_code' === $key && isset( $data['billing_wcexd_pec'] ) && null ==  $data['billing_wcexd_pec']  ) {
                    if ( null == $data[ $key ] ) {

                        $pa_code      = 'IT' === $data['billing_country'] ? '0000000' : 'xxxxxxx';
                        $data[ $key ] = $pa_code;

                    }

                }

				if ( isset( $data[ $key ] ) ) {
					$order->update_meta_data( '_' . $key, sanitize_text_field( $data[ $key ] ) );
				}
			}
		}

	}


	/**
	 * Visualizza le informazioni personalizzate nella pagina di checkout e in front-end, nell'area profilo dell'utente.
	 *
	 * @param  int $order_id   l'id dell'ordine.
	 */
	public function display_custom_data( $order_id ) {

		if ( $this->custom_fields ) {

			$order = wc_get_order( $order_id );

			echo '<h2>' . esc_html__( 'Dati di fatturazione', 'wcexd' ) . '</h2>';

			echo '<table class="shop_table shop_table_responsive">';
				echo '<tbody>';
			if ( $this->custom_fields ) {
				foreach ( $this->custom_fields as $key => $value ) {
					if ( $order->get_meta( '_' . $key ) ) {
						echo '<tr>';
							echo '<th width="40%">' . esc_html( $value ) . ':</th>';
							echo '<td>' . esc_html( $order->get_meta( '_' . $key ) ) . '</td>';
						echo '</tr>';
					}
				}
			}
				echo '</tbody>';
			echo '</table>';

		}
	}


	/**
	 * Visualizza le informazioni personalizzate nel back-end dell'ordine
	 *
	 * @param  object $order l'ordine.
	 */
	public function display_custom_data_in_admin( $order ) {

		if ( $this->custom_fields ) {

			foreach ( $this->custom_fields as $key => $value ) {
				if ( $order->get_meta( '_' . $key ) ) {
					echo '<p><strong>' . esc_html( $value ) . ': </strong><br>' . esc_html( $order->get_meta( '_' . $key ) ) . '</p>';
				}
			}

		}
	}


	/**
	 * Mostra le informaizoni personalizzate nella email di conferma ordine
	 *
	 * @param object $order l'ordine WC.
	 */
	public function display_custom_data_in_email( $order ) {

		if ( $this->custom_fields ) {

			echo '<h2>' . esc_html__( 'Dati fatturazione', 'wcexd' ) . '</h2>';
			foreach ( $this->custom_fields as $key => $value ) {
				if ( $order->get_meta( '_' . $key ) ) {
					echo '<p style="margin: 0 0 8px;">' . esc_html( $value ) . ': <span style="font-weight: normal;">' . esc_html( $order->get_meta( '_' . $key ) ) . '</span></p>';
				}
			}
			echo '<div style="display: block; padding-bottom: 25px;"></div>';
		}

	}


	/**
	 * Aggiunge i campi fiscali del plugin alla pagina profilo dell'utente
	 *
	 * @param  object $user user WP.
	 */
	public function extra_user_profile_fields( $user ) {

		if ( $this->custom_fields ) {

			echo '<h3>' . esc_html__( 'Dati di fatturazione', 'wcexd' ) . '</h3>';

			echo '<table class="form-table">';

				foreach ( $this->custom_fields as $key => $value ) {

					echo '<tr>';
						echo '<th><label for="' . esc_attr( $key ) . '">' . esc_html( $value ) . '</label></th>';
						echo '<td>';
							echo '<input type="text" name="' . esc_attr( $key ) . '" id="' . esc_attr( $key ) . '" value="' . esc_attr( get_the_author_meta( $key, $user->ID ) ) . '" class="regular-text" />';
						echo '</td>';
					echo '</tr>';

				}

			echo '</table>';
		}

	}

	/**
	 * Consente di editare i campi fiscali del plugin nella pagina profilo dell'utente
	 *
	 * @param  int $user_id l'id dell'utente WP.
	 */
	public function save_extra_user_profile_fields( $user_id ) {

		if ( ! current_user_can( 'edit_user', $user_id ) ) {

			return false;

		} else {

			if ( $this->custom_fields ) {

				foreach ( $this->custom_fields as $key => $value ) {

					$new_value = isset( $_POST[ $key ] ) ? sanitize_text_field( wp_unslash( $_POST[ $key ] ) ) : '';

					if ( $new_value ) {

						update_user_meta( $user_id, $key, $new_value );

					}

				}

			}

		}

	}

}
new WCEXD_Checkout_Fields();
